@@ -1,55 +1,92 @@
 import abc
-import textwrap
 import numbers
+import textwrap
+from copy import deepcopy
 
-import numpy as np
-from astropy.time import TimeDelta
 import astropy.units as u
+import ndcube.utils.wcs as nuw
+import numpy as np
+from astropy.time import Time
 from ndcube.ndcube import NDCube
-from ndcube.utils.cube import convert_extra_coords_dict_to_input_format
 
-__all__ = ['SpectrogramCube']
+from sunraster.meta import Meta
+
+__all__ = ["SpectrogramCube"]
 
 
 # Define some custom error messages.
-APPLY_EXPOSURE_TIME_ERROR = ("Exposure time correction has probably already "
-                             "been applied since the unit already includes "
-                             "inverse time. To apply exposure time correction "
-                             "anyway, set 'force' kwarg to True.")
-UNDO_EXPOSURE_TIME_ERROR = ("Exposure time correction has probably already "
-                            "been undone since the unit does not include "
-                            "inverse time. To undo exposure time correction "
-                            "anyway, set 'force' kwarg to True.")
-AXIS_NOT_FOUND_ERROR = " axis not found. If in extra_coords, axis name must be supported: "
+APPLY_EXPOSURE_TIME_ERROR = (
+    "Exposure time correction has probably already "
+    "been applied since the unit already includes "
+    "inverse time. To apply exposure time correction "
+    "anyway, set 'force' kwarg to True."
+)
+UNDO_EXPOSURE_TIME_ERROR = (
+    "Exposure time correction has probably already "
+    "been undone since the unit does not include "
+    "inverse time. To undo exposure time correction "
+    "anyway, set 'force' kwarg to True."
+)
+AXIS_NOT_FOUND_ERROR = "axis not found. If in extra_coords, axis name must be supported:"
 
 # Define supported coordinate names for coordinate properties.
-SUPPORTED_LONGITUDE_NAMES = ["custom:pos.helioprojective.lon", "pos.helioprojective.lon",
-                             "longitude", "lon"]
-SUPPORTED_LONGITUDE_NAMES += [name.upper() for name in SUPPORTED_LONGITUDE_NAMES] + \
-                             [name.capitalize() for name in SUPPORTED_LONGITUDE_NAMES]
+SUPPORTED_LONGITUDE_NAMES = [
+    "custom:pos.helioprojective.lon",
+    "pos.helioprojective.lon",
+    "longitude",
+    "lon",
+]
+SUPPORTED_LONGITUDE_NAMES += [name.upper() for name in SUPPORTED_LONGITUDE_NAMES] + [
+    name.capitalize() for name in SUPPORTED_LONGITUDE_NAMES
+]
 SUPPORTED_LONGITUDE_NAMES = np.array(SUPPORTED_LONGITUDE_NAMES)
 
-SUPPORTED_LATITUDE_NAMES = ["custom:pos.helioprojective.lat", "pos.helioprojective.lat",
-                            "latitude", "lat"]
-SUPPORTED_LATITUDE_NAMES += [name.upper() for name in SUPPORTED_LATITUDE_NAMES] + \
-                            [name.capitalize() for name in SUPPORTED_LATITUDE_NAMES]
+SUPPORTED_LATITUDE_NAMES = [
+    "custom:pos.helioprojective.lat",
+    "pos.helioprojective.lat",
+    "latitude",
+    "lat",
+]
+SUPPORTED_LATITUDE_NAMES += [name.upper() for name in SUPPORTED_LATITUDE_NAMES] + [
+    name.capitalize() for name in SUPPORTED_LATITUDE_NAMES
+]
 SUPPORTED_LATITUDE_NAMES = np.array(SUPPORTED_LATITUDE_NAMES)
 
-SUPPORTED_SPECTRAL_NAMES = ["em.wl", "em.energy", "em.freq", "wavelength", "energy",
-                            "frequency", "freq", "lambda", "spectral"]
-SUPPORTED_SPECTRAL_NAMES += [name.upper() for name in SUPPORTED_SPECTRAL_NAMES] + \
-                            [name.capitalize() for name in SUPPORTED_SPECTRAL_NAMES]
+SUPPORTED_SPECTRAL_NAMES = [
+    "em.wl",
+    "em.energy",
+    "em.freq",
+    "wavelength",
+    "energy",
+    "frequency",
+    "freq",
+    "lambda",
+    "spectral",
+]
+SUPPORTED_SPECTRAL_NAMES += [name.upper() for name in SUPPORTED_SPECTRAL_NAMES] + [
+    name.capitalize() for name in SUPPORTED_SPECTRAL_NAMES
+]
 SUPPORTED_SPECTRAL_NAMES = np.array(SUPPORTED_SPECTRAL_NAMES)
 
 SUPPORTED_TIME_NAMES = ["time"]
-SUPPORTED_TIME_NAMES += [name.upper() for name in SUPPORTED_TIME_NAMES] + \
-                        [name.capitalize() for name in SUPPORTED_TIME_NAMES]
+SUPPORTED_TIME_NAMES += [name.upper() for name in SUPPORTED_TIME_NAMES] + [
+    name.capitalize() for name in SUPPORTED_TIME_NAMES
+]
 SUPPORTED_TIME_NAMES = np.array(SUPPORTED_TIME_NAMES)
 
-SUPPORTED_EXPOSURE_NAMES = ["exposure time", "exposure_time", "exposure times",
-                            "exposure_times", "exp time", "exp_time", "exp times", "exp_times"]
-SUPPORTED_EXPOSURE_NAMES += [name.upper() for name in SUPPORTED_EXPOSURE_NAMES] + \
-                            [name.capitalize() for name in SUPPORTED_EXPOSURE_NAMES]
+SUPPORTED_EXPOSURE_NAMES = [
+    "exposure time",
+    "exposure_time",
+    "exposure times",
+    "exposure_times",
+    "exp time",
+    "exp_time",
+    "exp times",
+    "exp_times",
+]
+SUPPORTED_EXPOSURE_NAMES += [name.upper() for name in SUPPORTED_EXPOSURE_NAMES] + [
+    name.capitalize() for name in SUPPORTED_EXPOSURE_NAMES
+]
 SUPPORTED_EXPOSURE_NAMES = np.array(SUPPORTED_EXPOSURE_NAMES)
 
 
@@ -76,15 +113,9 @@ class SpectrogramABC(abc.ABC):
         """
 
     @abc.abstractproperty
-    def lon(self):
+    def celestial(self):
         """
-        Return the longitude coordinates for each pixel.
-        """
-
-    @abc.abstractproperty
-    def lat(self):
-        """
-        Return the latitude coordinates for each pixel.
+        Return the celestial coordinates for each pixel.
         """
 
     @abc.abstractmethod
@@ -94,7 +125,7 @@ class SpectrogramABC(abc.ABC):
         adjusts unit.
 
         Correction is only applied (undone) if the object's unit doesn't (does)
-        already include inverse time.  This can be overridden so that correction
+        already include inverse time. This can be overridden so that correction
         is applied (undone) regardless of unit by setting force=True.
 
         Parameters
@@ -103,11 +134,10 @@ class SpectrogramABC(abc.ABC):
             If False, exposure time correction is applied.
             If True, exposure time correction is undone.
             Default=False
-
         force: `bool`
             If not True, applies (undoes) exposure time correction only if unit
             doesn't (does) already include inverse time.
-            If True, correction is applied (undone) regardless of unit.  Unit is still
+            If True, correction is applied (undone) regardless of unit. Unit is still
             adjusted accordingly.
 
         Returns
@@ -128,34 +158,27 @@ class SpectrogramCube(NDCube, SpectrogramABC):
     ----------
     data: `numpy.ndarray`
         The array holding the actual data in this object.
-
     wcs: `ndcube.wcs.wcs.WCS`
         The WCS object containing the axes' information
-
-    extra_coords : iterable of `tuple`, each with three entries, optional
-        (`str`, `int`, `astropy.units.quantity` or array-like)
-        Gives the name, axis of data, and values of coordinates of a data axis not
-        included in the WCS object.
-
     unit : `astropy.unit.Unit` or `str`, optional
         Unit for the dataset. Strings that can be converted to a Unit are allowed.
-
-    meta : dict-like object, optional
-        Additional meta information about the dataset.
-
     uncertainty : any type, optional
         Uncertainty in the dataset. Should have an attribute uncertainty_type
         that defines what kind of uncertainty is stored, for example "std"
         for standard deviation or "var" for variance. A metaclass defining
-        such an interface is NDUncertainty - but isn’t mandatory. If the uncertainty
+        such an interface is NDUncertainty - but isn't mandatory. If the uncertainty
         has no such attribute the uncertainty is stored as UnknownUncertainty.
         Defaults to None.
-
+    meta : dict-like object, optional
+        Additional meta information about the dataset.
     mask : any type, optional
         Mask for the dataset. Masks should follow the numpy convention
         that valid data points are marked by False and invalid ones with True.
         Defaults to None.
-
+    instrument_axes : list, optional
+        This is the relationship between the array axes and the instrument,
+        i.e. repeat raster axis, slit position, position along slit, and spectral.
+        These are needed because they cannot be inferred simply from the physical types.
     copy : `bool`, optional
         Indicates whether to save the arguments as copy. True copies every attribute
         before saving it while False tries to save every parameter as reference.
@@ -163,26 +186,62 @@ class SpectrogramCube(NDCube, SpectrogramABC):
         Default is False.
     """
 
-    def __init__(self, data, wcs, extra_coords=None, unit=None, uncertainty=None, meta=None,
-                 mask=None, instrument_axes=None, copy=False, **kwargs):
+    def __init__(
+        self,
+        data,
+        wcs,
+        unit=None,
+        uncertainty=None,
+        meta=None,
+        mask=None,
+        instrument_axes=None,
+        copy=False,
+        **kwargs,
+    ):
         # Initialize SpectrogramCube.
-        super().__init__(data, wcs=wcs, uncertainty=uncertainty, mask=mask, meta=meta,
-                         unit=unit, extra_coords=extra_coords, copy=copy, **kwargs)
-
+        super().__init__(
+            data,
+            wcs=wcs,
+            uncertainty=uncertainty,
+            mask=mask,
+            meta=meta,
+            unit=unit,
+            copy=copy,
+            **kwargs,
+        )
         # Determine labels and location of each key real world coordinate.
         self_extra_coords = self.extra_coords
         world_axis_physical_types = np.array(self.wcs.world_axis_physical_types)
         self._longitude_name, self._longitude_loc = _find_axis_name(
-            SUPPORTED_LONGITUDE_NAMES, world_axis_physical_types, self_extra_coords)
+            SUPPORTED_LONGITUDE_NAMES,
+            world_axis_physical_types,
+            self_extra_coords,
+            self.meta,
+        )
         self._latitude_name, self._latitude_loc = _find_axis_name(
-            SUPPORTED_LATITUDE_NAMES, world_axis_physical_types, self_extra_coords)
+            SUPPORTED_LATITUDE_NAMES,
+            world_axis_physical_types,
+            self_extra_coords,
+            self.meta,
+        )
         self._spectral_name, self._spectral_loc = _find_axis_name(
-            SUPPORTED_SPECTRAL_NAMES, world_axis_physical_types, self_extra_coords)
+            SUPPORTED_SPECTRAL_NAMES,
+            world_axis_physical_types,
+            self_extra_coords,
+            self.meta,
+        )
         self._time_name, self._time_loc = _find_axis_name(
-            SUPPORTED_TIME_NAMES, world_axis_physical_types, self_extra_coords)
+            SUPPORTED_TIME_NAMES,
+            world_axis_physical_types,
+            self_extra_coords,
+            self.meta,
+        )
         self._exposure_time_name, self._exposure_time_loc = _find_axis_name(
-            SUPPORTED_EXPOSURE_NAMES, world_axis_physical_types, self_extra_coords)
-
+            SUPPORTED_EXPOSURE_NAMES,
+            world_axis_physical_types,
+            self_extra_coords,
+            self.meta,
+        )
         # Set up instrument axes if set.
         if instrument_axes is None:
             self.instrument_axes = instrument_axes
@@ -190,37 +249,54 @@ class SpectrogramCube(NDCube, SpectrogramABC):
             raise ValueError("Length of instrument_axes must match number of data axes.")
         else:
             self.instrument_axes = np.asarray(instrument_axes, dtype=str)
+        # TODO: Remove after ndcube 2.1
+        if not isinstance(self.meta, Meta):
+            self.meta = Meta(self.meta, data_shape=self.data.shape)
 
     def __str__(self):
-        if self._time_name:
+        try:
             if self.time.isscalar:
                 time_period = self.time
             else:
-                time_period = (str(self.time.min()), str(self.time.max()))
-        else:
-            time_period = None
-        if self._longitude_name:
-            if self.lon.isscalar:
-                lon_range = self.lon
+                times = self.time
+                time_period = Time([times.min(), times.max()]).iso
+        except ValueError as err:
+            if AXIS_NOT_FOUND_ERROR in err.args[0]:
+                time_period = None
             else:
-                lon_range = u.Quantity([self.lon.min(), self.lon.max()])
-        else:
-            lon_range = None
-        if self._latitude_name:
-            if self.lat.isscalar:
-                lat_range = self.lat
+                raise err
+        try:
+            sc = self.celestial
+            component_names = dict([(item, key) for key, item in sc.representation_component_names.items()])
+            lon = getattr(sc, component_names["lon"])
+            lat = getattr(sc, component_names["lat"])
+            if sc.isscalar:
+                lon_range = lon
+                lat_range = lat
+            elif sc.size == 0:
+                lon_range = None
+                lat_range = None
             else:
-                lat_range = u.Quantity([self.lat.min(), self.lat.max()])
-        else:
-            lat_range = None
-        if self._spectral_name:
+                lon_range = u.Quantity([lon.min(), lon.max()])
+                lat_range = u.Quantity([lat.min(), lat.max()])
+        except ValueError as err:
+            if AXIS_NOT_FOUND_ERROR in err.args[0]:
+                lon_range = None
+                lat_range = None
+            else:
+                raise err
+        try:
             if self.spectral_axis.isscalar:
                 spectral_range = self.spectral_axis
             else:
                 spectral_range = u.Quantity([self.spectral_axis.min(), self.spectral_axis.max()])
-        else:
-            spectral_range = None
-        return (textwrap.dedent(f"""\
+        except ValueError as err:
+            if AXIS_NOT_FOUND_ERROR in err.args[0]:
+                spectral_range = None
+            else:
+                raise err
+        return textwrap.dedent(
+            f"""\
                 {self.__class__.__name__}
                 {"".join(["-"] * len(self.__class__.__name__))}
                 Time Period: {time_period}
@@ -229,7 +305,8 @@ class SpectrogramCube(NDCube, SpectrogramABC):
                 Longitude range: {lon_range}
                 Latitude range: {lat_range}
                 Spectral range: {spectral_range}
-                Data unit: {self.unit}"""))
+                Data unit: {self.unit}"""
+        )
 
     def __repr__(self):
         return f"{object.__repr__(self)}\n{str(self)}"
@@ -237,15 +314,6 @@ class SpectrogramCube(NDCube, SpectrogramABC):
     def __getitem__(self, item):
         # Slice SpectrogramCube using parent slicing.
         result = super().__getitem__(item)
-
-        # As result is an NDCube, must put extra coord back into input format
-        # to create a SpectrogramCube.
-        if result.extra_coords is None:
-            extra_coords = None
-        else:
-            extra_coords = convert_extra_coords_dict_to_input_format(result.extra_coords,
-                                                                     result.missing_axes)
-
         # Slice instrument_axes if it exists.
         # If item is a slice, cube dimensionality is not reduced
         # so instrument_axes need not be sliced.
@@ -257,61 +325,94 @@ class SpectrogramCube(NDCube, SpectrogramABC):
                 instrument_axes = self.instrument_axes[1:]
             # If item is tuple, instrument axes will need to be sliced if tuple contains an int.
             elif isinstance(item, tuple):
-                instr_item = [isinstance(i, numbers.Integral) for i in item] + \
-                        [False] * (len(self.instrument_axes) - len(item))
+                instr_item = [isinstance(i, numbers.Integral) for i in item] + [False] * (
+                    len(self.instrument_axes) - len(item)
+                )
                 instrument_axes = self.instrument_axes[np.invert(instr_item)]
             else:
                 raise TypeError("Unrecognized slice item. Must be int, slice or tuple.")
             # If slicing causes cube to be a scalar, set instrument_axes to None.
             if len(instrument_axes) == 0:
                 instrument_axes = None
-
-        return self.__class__(result.data, wcs=result.wcs, uncertainty=result.uncertainty,
-                              mask=result.mask, meta=result.meta, unit=result.unit,
-                              extra_coords=extra_coords, missing_axes=result.missing_axes,
-                              instrument_axes=instrument_axes)
+        result.instrument_axes = instrument_axes
+        # TODO: Remove for ndcube 2.1
+        # Slice metadata if possible.
+        try:
+            result.meta = self.meta[item]
+        except TypeError as err:
+            if "unhashable type" not in err.args[0]:
+                raise err
+        except KeyError:
+            pass
+        return result
 
     @property
     def spectral_axis(self):
         if not self._spectral_name:
-            raise ValueError("Spectral" + AXIS_NOT_FOUND_ERROR +
-                             f"{SUPPORTED_SPECTRAL_NAMES}")
+            self._spectral_name, self._spectral_loc = _find_axis_name(
+                SUPPORTED_SPECTRAL_NAMES,
+                self.wcs.world_axis_physical_types,
+                self.extra_coords,
+                self.meta,
+            )
+        if not self._spectral_name:
+            raise ValueError("Spectral" + AXIS_NOT_FOUND_ERROR + f"{SUPPORTED_SPECTRAL_NAMES}")
         return self._get_axis_coord(self._spectral_name, self._spectral_loc)
 
     @property
     def time(self):
         if not self._time_name:
-            raise ValueError("Time" + AXIS_NOT_FOUND_ERROR +
-                             f"{SUPPORTED_TIME_NAMES}")
-        times = self._get_axis_coord(self._time_name, self._time_loc)
-        if isinstance(times, (u.Quantity, TimeDelta)):
-            try:
-                if self.meta.date_reference:
-                    times += self.meta.date_reference
-            except AttributeError:
-                pass
-        return times
+            self._time_name, self._time_loc = _find_axis_name(
+                SUPPORTED_TIME_NAMES,
+                self.wcs.world_axis_physical_types,
+                self.extra_coords,
+                self.meta,
+            )
+            if not self._time_name:
+                raise ValueError(f"Time {AXIS_NOT_FOUND_ERROR} {SUPPORTED_TIME_NAMES}")
+        return Time(self._get_axis_coord(self._time_name, self._time_loc))
 
     @property
     def exposure_time(self):
-        if not self._exposure_time_name:
-            raise ValueError("Exposure time" + AXIS_NOT_FOUND_ERROR +
-                             f"{SUPPORTED_EXPOSURE_NAMES}")
+        if not self._exposure_time_name or not hasattr(self, "_exposure_time_loc"):
+            self._exposure_time_name, self._exposure_time_loc = _find_axis_name(
+                SUPPORTED_EXPOSURE_NAMES,
+                self.wcs.world_axis_physical_types,
+                self.extra_coords,
+                self.meta,
+            )
+            if not self._exposure_time_name:
+                raise ValueError(f"Exposure time {AXIS_NOT_FOUND_ERROR} {SUPPORTED_EXPOSURE_NAMES}")
         return self._get_axis_coord(self._exposure_time_name, self._exposure_time_loc)
 
     @property
-    def lon(self):
+    def celestial(self):
         if not self._longitude_name:
-            raise ValueError("Longitude" + AXIS_NOT_FOUND_ERROR +
-                             f"{SUPPORTED_LONGITUDE_NAMES}")
-        return self._get_axis_coord(self._longitude_name, self._longitude_loc)
-
-    @property
-    def lat(self):
+            self._longitude_name, self._longitude_loc = _find_axis_name(
+                SUPPORTED_LONGITUDE_NAMES,
+                self.wcs.world_axis_physical_types,
+                self.extra_coords,
+                self.meta,
+            )
         if not self._latitude_name:
-            raise ValueError("Latitude" + AXIS_NOT_FOUND_ERROR +
-                             f"{SUPPORTED_LATITUDE_NAMES}")
-        return self._get_axis_coord(self._latitude_name, self._latitude_loc)
+            self._latitude_name, self._latitude_loc = _find_axis_name(
+                SUPPORTED_LATITUDE_NAMES,
+                self.wcs.world_axis_physical_types,
+                self.extra_coords,
+                self.meta,
+            )
+        if self._longitude_name:
+            celestial_name = self._longitude_name
+            celestial_loc = self._longitude_loc
+        elif self._latitude_name:
+            celestial_name = self._latitude_name
+            celestial_loc = self._latitude_loc
+        else:
+            raise ValueError(
+                f"Celestial {AXIS_NOT_FOUND_ERROR} "
+                f"{np.concatenate([SUPPORTED_LONGITUDE_NAMES, SUPPORTED_LATITUDE_NAMES])}"
+            )
+        return self._get_axis_coord(celestial_name, celestial_loc)
 
     def apply_exposure_time_correction(self, undo=False, force=False):
         # Get exposure time in seconds.
@@ -319,12 +420,7 @@ class SpectrogramCube(NDCube, SpectrogramABC):
         # If exposure time is not scalar, change array's shape so that
         # it can be broadcast with data and uncertainty arrays.
         if not np.isscalar(exposure_time_s):
-            # Get data axis of exposure time.
-            if self._exposure_time_loc == "extra_coords":
-                exposure_axis = self.extra_coords[self._exposure_time_name]["axis"]
-            else:
-                exposure_axis = get_axis_number_from_axis_name(self._exposure_time_name,
-                                                               self.world_axis_physical_types)
+            (exposure_axis,) = self._get_axis_coord_index(self._exposure_time_name, self._exposure_time_loc)
             # Change array shape for broadcasting
             item = [np.newaxis] * self.data.ndim
             item[exposure_axis] = slice(None)
@@ -332,27 +428,56 @@ class SpectrogramCube(NDCube, SpectrogramABC):
         # Based on value on undo kwarg, apply or remove exposure time correction.
         if undo is True:
             new_data, new_uncertainty, new_unit = _uncalculate_exposure_time_correction(
-                self.data, self.uncertainty, self.unit, exposure_time_s, force=force)
+                self.data, self.uncertainty, self.unit, exposure_time_s, force=force
+            )
         else:
             new_data, new_uncertainty, new_unit = _calculate_exposure_time_correction(
-                self.data, self.uncertainty, self.unit, exposure_time_s, force=force)
+                self.data, self.uncertainty, self.unit, exposure_time_s, force=force
+            )
         # Return new instance of SpectrogramCube with correction applied/undone.
-
-        return self.__class__(
-            new_data, wcs=self.wcs,
-            extra_coords=convert_extra_coords_dict_to_input_format(self.extra_coords,
-                                                                   self.missing_axes),
-            unit=new_unit, uncertainty=new_uncertainty, meta=self.meta, mask=self.mask,
-            missing_axes=self.missing_axes)
+        new_cube = deepcopy(self)
+        new_cube._data = new_data
+        new_cube._uncertainty = new_uncertainty
+        new_cube._extra_coords = self.extra_coords
+        new_cube._unit = new_unit
+        return new_cube
 
     def _get_axis_coord(self, axis_name, coord_loc):
+        if coord_loc == "wcs":
+            return self.axis_world_coords(axis_name)[0]
+        elif coord_loc == "extra_coords":
+            return self.axis_world_coords(wcs=self.extra_coords[axis_name])[0]
+        elif coord_loc == "global_coords":
+            return self.global_coords[axis_name]
+        elif coord_loc == "meta":
+            return self.meta[axis_name]
+        else:
+            raise ValueError(f"{coord_loc} is not a valid coordinate location.")
+
+    def _get_axis_coord_index(self, axis_name, coord_loc):
+        if coord_loc == "wcs":
+            coord_pix_axes = nuw.physical_type_to_pixel_axes(axis_name, self.wcs)
+            coord_array_axes = nuw.convert_between_array_and_pixel_axes(coord_pix_axes, len(self.dimensions))
+            return coord_array_axes.tolist()[0]
+        elif coord_loc == "extra_coords":
+            return self.extra_coords[axis_name].mapping[0]
+        elif coord_loc == "meta":
+            return self.meta.axes[axis_name]
+        else:
+            raise ValueError(f"{coord_loc} is not a valid coordinate location.")
+
+    def _get_axis_coord_values(self, axis_name, coord_loc):
         if coord_loc == "wcs":
             return self.axis_world_coords_values(axis_name)[0]
         elif coord_loc == "extra_coords":
-            return self.extra_coords[axis_name]["value"]
+            return self.axis_world_coords_values(wcs=self.extra_coords[axis_name])[0]
+        elif coord_loc == "global_coords":
+            return self.global_coords[axis_name]
+        else:
+            raise ValueError(f"{coord_loc} is not a valid coordinate location.")
 
 
-def _find_axis_name(supported_names, world_axis_physical_types, extra_coords):
+def _find_axis_name(supported_names, world_axis_physical_types, extra_coords, meta):
     """
     Finds name of a SpectrogramCube axis type from WCS and extra coords.
 
@@ -360,18 +485,17 @@ def _find_axis_name(supported_names, world_axis_physical_types, extra_coords):
     ----------
     supported_names: 1D `numpy.ndarray`
         The names for the axis supported by `SpectrogramCube`.
-
     world_axis_physical_types: 1D `numpy.ndarray`
         Output of SpectrogramCube.world_axis_physical_types converted to an array.
-
-    extra_coords: `dict` or `None`
+    extra_coords: `ndcube.ExtraCoords` or `None`
         Output of SpectrogramCube.extra_coords
+    meta: Meta or `None`
+        Output of SpectrogramCube.meta
 
     Returns
     -------
     axis_name: `str`
         The coordinate name of the axis.
-
     loc: `str`
         The location where the coordinate is stored: "wcs" or "extra_coords".
     """
@@ -385,6 +509,10 @@ def _find_axis_name(supported_names, world_axis_physical_types, extra_coords):
         axis_name = _find_name_in_array(supported_names, np.array(list(extra_coords.keys())))
         if axis_name:
             loc = "extra_coords"
+    if axis_name is None and meta:  # If axis name not in WCS, check meta.
+        axis_name = _find_name_in_array(supported_names, np.array(list(meta.keys())))
+        if axis_name:
+            loc = "meta"
     return axis_name, loc
 
 
@@ -393,6 +521,7 @@ def _find_name_in_array(supported_names, names_array):
     if name_index.sum() > 0:
         name_index = int(np.arange(len(names_array))[name_index])
         return names_array[name_index]
+    return None
 
 
 def _calculate_exposure_time_correction(data, uncertainty, unit, exposure_time, force=False):
@@ -403,13 +532,10 @@ def _calculate_exposure_time_correction(data, uncertainty, unit, exposure_time,
     ----------
     data: `numpy.ndarray`
         Data array to be converted.
-
     uncertainty: `astropy.nddata.nduncertainty.NDUncertainty`
         The uncertainty of each element in data.
-
     old_unit: `astropy.unit.Unit`
         Unit of data arrays.
-
     exposure_time: `numpy.ndarray`
         Exposure time in seconds for each exposure in data arrays.
 
@@ -417,10 +543,8 @@ def _calculate_exposure_time_correction(data, uncertainty, unit, exposure_time,
     -------
     new_data: `numpy.ndarray`
         Data array with exposure time corrected for.
-
     new_uncertainty: `astropy.nddata.nduncertainty.NDUncertainty`
         The uncertainty of each element in new_data.
-
     new_unit: `astropy.unit.Unit`
         Unit of new_data array after exposure time correction.
     """
@@ -438,8 +562,7 @@ def _calculate_exposure_time_correction(data, uncertainty, unit, exposure_time,
         new_data = data / exposure_time
         if uncertainty:
             uncertainty_unit = uncertainty.unit / u.s if uncertainty.unit else uncertainty.unit
-            new_uncertainty = uncertainty.__class__(uncertainty.array / exposure_time,
-                                                    unit=uncertainty_unit)
+            new_uncertainty = uncertainty.__class__(uncertainty.array / exposure_time, unit=uncertainty_unit)
         else:
             new_uncertainty = uncertainty
         new_unit = unit / u.s
@@ -454,13 +577,10 @@ def _uncalculate_exposure_time_correction(data, uncertainty, unit, exposure_time
     ----------
     data: `numpy.ndarray`
         Data array to be converted.
-
     uncertainty: `astropy.nddata.nduncertainty.NDUncertainty`
         The uncertainty of each element in data.
-
     old_unit: `astropy.unit.Unit`
         Unit of data arrays.
-
     exposure_time: `numpy.ndarray`
         Exposure time in seconds for each exposure in data arrays.
 
@@ -468,10 +588,8 @@ def _uncalculate_exposure_time_correction(data, uncertainty, unit, exposure_time
     -------
     new_data: `numpy.ndarray`
         Data array with exposure time corrected for.
-
     new_uncertainty: `astropy.nddata.nduncertainty.NDUncertainty`
         The uncertainty of each element in new_data.
-
     new_unit: `astropy.unit.Unit`
         Unit of new_data array after exposure time correction.
     """
@@ -489,8 +607,7 @@ def _uncalculate_exposure_time_correction(data, uncertainty, unit, exposure_time
         new_data = data * exposure_time
         if uncertainty:
             uncertainty_unit = uncertainty.unit * u.s if uncertainty.unit else uncertainty.unit
-            new_uncertainty = uncertainty.__class__(uncertainty.array * exposure_time,
-                                                    unit=uncertainty_unit)
+            new_uncertainty = uncertainty.__class__(uncertainty.array * exposure_time, unit=uncertainty_unit)
         else:
             new_uncertainty = uncertainty
         new_unit = unit * u.s
